package com.persagy.admp

import io.swagger.v3.oas.annotations.enums.ParameterIn
import io.swagger.v3.oas.models.*
import io.swagger.v3.oas.models.info.Contact
import io.swagger.v3.oas.models.info.Info
import io.swagger.v3.oas.models.info.License
import io.swagger.v3.oas.models.media.StringSchema
import io.swagger.v3.oas.models.parameters.Parameter
import io.swagger.v3.oas.models.security.SecurityScheme
import org.springdoc.core.GroupedOpenApi
import org.springdoc.core.customizers.OperationCustomizer
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.web.method.HandlerMethod

/**
 * API 文档生成器 Swagger3 配置
 *
 * @author  张维新 <zhangweixin@sagacloud.cn>
 */
 @Configuration
open class WebApiConfig {
    /** 标题 */
    private val title = "建筑信息管理平台"
    /** API 版本号 */
    private val version = "2.0"

    private val operationCustomizer = GlobalHeaderOperationCustomizer()

    /**
     * API 信息
     */
    @Bean
    open fun openApi(): OpenAPI {
        val contact = Contact().name("张维新").email("zhangweixin@sagacloud.cn")
        val projectId = Parameter().`in`(ParameterIn.HEADER.toString())
                .name("projectId").description("项目 id")
                .schema(StringSchema()).required(false)

        val api = OpenAPI()
                .components(Components()
                        .addSecuritySchemes("basicScheme", SecurityScheme().type(SecurityScheme.Type.HTTP).scheme("basic")))
                .info(Info()
                        .title(title)
                        .version(version)
                        .termsOfService("http://www.persagy.com")
                        .contact(contact)
                        .license(License()
                                .name("Apache 2.0")
                                .url("http://www.persagy.com")))

        api.addExtension("abc", "dd")
        return api
    }

    /**
     * 全部 API 组
     *
     * @return 全部 API 组
     */
    @Bean
    open fun allApiGroup(): GroupedOpenApi {
        return GroupedOpenApi.builder()
            .group("全部")
            .pathsToMatch("/**")
            .build()
    }

    /**
     * 字典 API 组
     *
     * @return 字典 API 组
     */
    @Bean
    open fun dictApiGroup(): GroupedOpenApi {
        return GroupedOpenApi.builder()
                .group("数据字典")
                .pathsToMatch("/dict/**")
                .build()
    }

    /**
     * 项目管理 API 组
     *
     * @return 项目 API 组
     */
    @Bean
    open fun pmApiGroup(): GroupedOpenApi {
        return GroupedOpenApi.builder()
                .group("项目管理")
                .pathsToMatch("/pm/**")
                .addOperationCustomizer(operationCustomizer)
                .build()
    }

    /**
     * 建筑 API 组 (architecture manage)
     *
     * @return  建筑 API 组
     */
    @Bean
    open fun amApiGroup(): GroupedOpenApi {
        return GroupedOpenApi.builder()
                .group("建筑模型")
                .pathsToMatch("/am/**")
                .addOperationCustomizer(operationCustomizer)
                .build()
    }

    /**
     *  http 请求头自定义
     *
     *  @author 张维新 <zhangweixin@sagacloud.cn>
     */
    class GlobalHeaderOperationCustomizer : OperationCustomizer {
        /**
         * 接口定义器
         *
         * @param   operation       接口
         * @param   handlerMethod   原接口方法
         *
         * @return  新定义的接口
         */
        override fun customize(operation: Operation, handlerMethod: HandlerMethod): Operation {
            val projectId = Parameter().`in`(ParameterIn.HEADER.toString())
                    .name("projectId").description("项目 id")
                    .schema(StringSchema()).required(false)
            operation.addParametersItem(projectId)
            return operation
        }
    }
}